package graph;
import java.util.*;

public class Node implements Comparable {
    protected Map properties = new TreeMap();
    protected Graph graph;
    protected ArrayList out = new ArrayList();
    protected ArrayList in = new ArrayList();

    protected void addOutEdge(Edge e) {
	out.add(e);
    }
    
    protected void addInEdge(Edge e) {
	in.add(e);
    }
    /**
     *Returns label of the node
     *@return a String containing the label
     */
    public String label() {
	return get("id");
    }
    
    public int compareTo(Object n) {
	return label().compareTo(((Node)n).label());
    }
    
    /**
     *Returns an ArrayList of the Nodes adjanced to the node.
     *Use this method with undirected graphs.
     *@return an ArrayList of the Nodes adjanced to the node
     */
    public ArrayList getAdj() {
	Set res = new TreeSet();
	for (int i=0; i < out.size(); i++) {
	    res.add(((Edge)out.get(i)).dest);
	}
	for (int i=0; i < in.size(); i++) {
	    res.add(((Edge)in.get(i)).source);
	}
	return new ArrayList(res);
    }

    public ArrayList getEdges() {
	Set res = new TreeSet();
	res.addAll(out);
	res.addAll(in);
	return new ArrayList(res);
    }



    /**
     *Returns an ArrayList of all the nodes to which there
     *exists an edge starting at the node
     *Use this method with directed graphs
     *@return an ArrayList of the nodes succeeding the node
     */
    public ArrayList getNext() {
	Set res = new TreeSet();
	for (int i=0; i < out.size(); i++)
	    res.add(out.get(i));
	return new ArrayList(res);
    }

    /**
     *Returns an ArrayList of edges that start from this node.
     *Use this method with directed graphs
     *@return an ArrayList containing out-edges
     */
    public ArrayList getOutEdges() {
	return out;
    }

    /**
     *Returns an ArrayList of all the nodes from which there
     *exists an edge ending at the node
     *Use this method with directed graphs
     *@return an ArrayList of the nodes preceding the node
     */
    public ArrayList getPrev() {
	Set res = new TreeSet();
	for (int i=0; i < in.size(); i++)
	    res.add(in.get(i));
	return new ArrayList(res);
    }

    /**
     *Returns an ArrayList of edges that end to this node.
     *Use this method with directed graphs
     *@return an ArrayList containing in-edges
     */
    public ArrayList getInEdges() {
	return in;
    }

    /**
     *Initializes a new Node belonging to graph given as argument with given label
     *@param graph Graph the node belongs to
     *@param label the label of the node
     */
    public Node(Graph graph, String label) {
	this.graph = graph;
	properties.put("id",label);
    }
    /**
     *Adds an edge starting from the node and ending to node given as argument
     *@param dst the destination node of the edge
     *@return the newly created edge 
     */
    public Edge addEdge(Node dst) {
	return graph.addEdge(this,dst);
    }
    /**
     *Sets the given property of the node to given value
     *@param property a String containing property for which the value is set
     *@param value a String containing the new value for the property
     */
    public void set(String property, String value) {
	properties.put(property,value);
    }

    /**
     *Sets the given property of the node to given value
     *@param property a String containing property for which the value is set
     *@param value an integer containing the new value for the property
     */
    public void setInt(String property, int value) {
	properties.put(property,String.valueOf(value));
    }
    /**
     *Sets the given property of the node to given value
     *@param property a String containing property for which the value is set
     *@param value an double containing the new value for the property
     */
    public void setDouble(String property, double value) {
	properties.put(property,String.valueOf(value));
    }

    /**
     *Returns the value for the given property of the node
     *@param property a String containing the requested property
     *@return a String containing the current value of the property, or null, if the property does not exist
     */
    public String get(String property) {
	return (String)properties.get(property);
    }

    /**
     *Returns the value for the given property of the node
     *@param property a String containing the requested property
     *@return an integer containing the current value of the property, or -1, if the property does not exist
     */
    public int getInt(String property) {
	try {
	    return Integer.parseInt((String)properties.get(property));
	} catch (Exception e) {
	    return -1;
	}
    }

    /**
     *Returns the value for the given property of the node
     *@param property a String containing the requested property
     *@return a double containing the current value of the property, or -1, if the property does not exist
     */
    public double getDouble(String property) {
	try {
	    return Double.parseDouble((String)properties.get(property));
	} catch (Exception e) {
	    return -1;
	}
    }

}
